-- AdjustCanHaveUpgrade
-- Author: Davii
-- DateCreated: 4/8/2013 12:30:45 PM
--------------------------------------------------------------

--Improvement
local iFarm = GameInfoTypes.IMPROVEMENT_FARM;
local iTradingPost = GameInfoTypes.IMPROVEMENT_TRADING_POST;
local iMine = GameInfoTypes.IMPROVEMENT_MINE;
local iQuarry = GameInfoTypes.IMPROVEMENT_QUARRY;
local iPlantation = GameInfoTypes.IMPROVEMENT_PLANTATION;

--DLC Improvement
local iPolder = GameInfoTypes.IMPROVEMENT_POLDER;
local iTerraceFarm = GameInfoTypes.IMPROVEMENT_TERRACE_FARM;
local iMoai = GameInfoTypes.IMPROVEMENT_MOAI;

--Upgrade
local iSuburb = GameInfoTypes.IMPROVEMENT_SUBURB;
local iEnhancedMine = GameInfoTypes.IMPROVEMENT_ENHANCED_MINE;
local iEnhancedQuarry = GameInfoTypes.IMPROVEMENT_ENHANCED_QUARRY;
local iTradingEstate = GameInfoTypes.IMPROVEMENT_TRADING_ESTATE;
local iFairTradePlantation = GameInfoTypes.IMPROVEMENT_FAIR_TRADE_PLANTATION;

--Upgrade Builds
local iBuildSuburb = GameInfoTypes.BUILD_SUBURB;
local iBuildEnhancedMine = GameInfoTypes.BUILD_ENHANCED_MINE;
local iBuildEnhancedQuarry = GameInfoTypes.BUILD_ENHANCED_QUARRY;
local iBuildTradingEstate = GameInfoTypes.BUILD_TRADING_ESTATE;
local iBuildFairTradePlantation = GameInfoTypes.BUILD_FAIR_TRADE_PLANTATION;

--Upgrade Required Techs
local iSuburbTech =  GameInfoTypes.TECH_ELECTRICITY;
local iEnhancedMineTech = GameInfoTypes.TECH_DYNAMITE;
local iEnhancedQuarryTech = GameInfoTypes.TECH_DYNAMITE;
local iTradingEstateTech = GameInfoTypes.TECH_COMBUSTION;
local iFairTradePlantationTech = TECH_GLOBALIZATION;

--------------------------------------------------------------
--------------------------------------------------------------

function UI_CheckUpgradeBuilds()

	print(" ***** RUNNING UPGRADE CITY CHECK ***** ");

	if (iPolder == nil) then
		iPolder = GameInfoTypes.IMPROVEMENT_FARM;
	end

	if (iTerraceFarm == nil) then
		iTerraceFarm = GameInfoTypes.IMPROVEMENT_FARM;
	end

	if (iMoai == nil) then
		iMoai = GameInfoTypes.IMPROVEMENT_FARM;
	end
	
	for iPlayerLoop = 0, GameDefines.MAX_CIV_PLAYERS-1, 1 do

		local pPlayer = Players[iPlayerLoop];
		local pTeam = Teams[pPlayer:GetTeam()];

		if (pPlayer:IsAlive()) then

			--IU_GetNumUpgradesDebug(iPlayerLoop, pPlayer); --debug function
			IU_CityCanUpgrade(iPlayerLoop, pPlayer, pTeam);

		end

	end

end

Events.ActivePlayerTurnStart.Add(UI_CheckUpgradeBuilds);

--------------------------------------------------------------

function IU_CityCanUpgrade(player, pPlayer, pTeam) --only taking care of cities, need to prevent plots outside of cities getting Suburbs/Estates
	
	local bHasSuburbTech = pTeam:GetTeamTechs():HasTech(iSuburbTech);
	local bHasTradingEstateTech = pTeam:GetTeamTechs():HasTech(iTradingEstateTech);
								
	for pCity in pPlayer:Cities() do

		local pCityPop = pCity:GetPopulation();
		
		local iSuburbsAllowed = (math.floor(pCityPop / 4));
		local iEstatesAllowed = (math.floor(pCityPop / 6));
		local iSuburbsBuilt = IU_GetNumSuburbs(pCity);
		local iEstatesBuilt = IU_GetNumEstates(pCity);
		
		for i = 1, pCity:GetNumCityPlots()-1, 1 do
    
			local cityPlot = pCity:GetCityIndexPlot(i);
			local plotImprovement = cityPlot:GetImprovementType();
			
			if (pCityPop < 12) then --doesn't appear to be considered (Trading Estates still being built without meeting pop or prior improvement requirement)

				cityPlot:CanHaveImprovement(iSuburb, pTeam, false);
				cityPlot:CanHaveImprovement(iTradingEstate, pTeam, false);
			
			else
				
				if (pCity:IsWorkingPlot(cityPlot) or pCity:CanWork(cityPlot) and plotImprovement ~= -1) then
				--if (pCity:CanWork(cityPlot) and plotImprovement ~= -1) then
				
					-- 2nd criteria (must already be improved, so that upgrading is progressive with city)
					if (plotImprovement == iFarm or plotImprovement == iPolder or plotImprovement == iTerraceFarm or plotImprovement == iTradingPost or plotImprovement == iMoai) then
						
						-- 3rd criteria Suburb
						if (bHasSuburbTech and iSuburbsAllowed > iSuburbsBuilt) then
							
							cityPlot:CanHaveImprovement(iSuburb, pTeam, true);
							cityPlot:CanBuild(iBuildSuburb, player, false);
							
						else

							cityPlot:CanHaveImprovement(iSuburb, pTeam, false);

						end

						-- 3rd criteria Trading Estate
						if (bHasTradingEstateTech and iEstatesAllowed > iEstatesBuilt) then

							cityPlot:CanHaveImprovement(iTradingEstate, pTeam, true);
							cityPlot:CanBuild(iBuildTradingEstate, player, true); --bool is test visible
							
						else

							cityPlot:CanHaveImprovement(iTradingEstate, pTeam, false);
						
						end

					else
					
						cityPlot:CanHaveImprovement(iSuburb, pTeam, false);
						cityPlot:CanHaveImprovement(iTradingEstate, pTeam, false);

					end
				
				else
				
					cityPlot:CanHaveImprovement(iSuburb, pTeam, false);
					cityPlot:CanHaveImprovement(iTradingEstate, pTeam, false);

				end
			end
	
		end
	end
end

--------------------------------------------------------------

function IU_GetNumSuburbs(pCity)

	local iNumSuburbs = 0;

	for i = 1, pCity:GetNumCityPlots()-1, 1 do

		local cityPlot = pCity:GetCityIndexPlot(i);
		local plotImprovement = cityPlot:GetImprovementType();
				
		if (plotImprovement == iSuburb) then
			iNumSuburbs = iNumSuburbs + 1;
		end

	end

	return iNumSuburbs;

end

--------------------------------------------------------------

function IU_GetNumEstates(pCity)

	local iNumEstates = 0;

	for i = 1, pCity:GetNumCityPlots()-1, 1 do

		local cityPlot = pCity:GetCityIndexPlot(i);
		local plotImprovement = cityPlot:GetImprovementType();

		if (plotImprovement == iTradingEstate) then
			iNumEstates = iNumEstates + 1;
		end

	end

	return iNumEstates;

end

--------------------------------------------------------------

function IU_GetNumUpgradesDebug(player, pPlayer)

	for pCity in pPlayer:Cities() do

		local pCityPop = pCity:GetPopulation();
		local iSuburbsAllowed = (math.floor(pCityPop / 4));
		local iEstatesAllowed = (math.floor(pCityPop / 6));
		local iSuburbsBuilt = IU_GetNumSuburbs(pCity);
		local iEstatesBuilt = IU_GetNumEstates(pCity);
			
		if (iSuburbsBuilt > iSuburbsAllowed) then
			print(" City over Suburb limit ");
		else
			print(string.format(" Player %i Suburbs allowed/built: %i (%i) ", player, iSuburbsAllowed, iSuburbsBuilt));
		end

		if (iEstatesBuilt > iEstatesAllowed) then
			print(" City over Estates limit ");
		else
			print(string.format(" Player %i Estates allowed/built: %i (%i) ", player, iEstatesAllowed, iEstatesBuilt));
		end
		
	end
end

--------------------------------------------------------------

UI_CheckUpgradeBuilds(); --so that criteria are present on load