-- ImprovementUpgrades
-- Author: Davii
-- DateCreated: 9/26/2012 5:08:20 PM
--------------------------------------------------------------
-- AMENDED VERSION OF REDOX REMOTE MINING UNITS LUA ROUTINE
--------------------------------------------------------------

WARN_NOT_SHARED = false;

include( "ShareData" );
include( "SaveUtils" );
include( "AdjustCanHaveUpgrade" );

MY_MOD_NAME = "4c756864-da74-4831-abc1-329b1150b9b4";

--------------------------------------------------------------

local aUpgradeInstallation = {};

local mapMaxX, mapMaxY = Map.GetGridSize();

--------------------------------------------------------------

function onImprovementUpgradeCreated(iHexX, iHexY)

	local pPlot = Map.GetPlot(ToGridFromHex(iHexX, iHexY));
	
	if (pPlot ~= nil) then
		
		local improvementType = pPlot:GetImprovementType();

		if (improvementType == GameInfoTypes.IMPROVEMENT_ENHANCED_MINE or improvementType == GameInfoTypes.IMPROVEMENT_FAIR_TRADE_PLANTATION) then

			local plotID = pPlot:GetX() + pPlot:GetY() * mapMaxX;

			local playerID = pPlot:GetOwner();
			
			local player = Players[playerID];
			local playerTeamID = player:GetTeam();

			--local playerTeamID = Game.GetActiveTeam();
			--local playerID = Game.GetActivePlayer();
			--local pPlayer = Players[playerID];
	
			local isPillaged = pPlot:IsImprovementPillaged();		
			
			if (isPillaged) then
			
				if( aUpgradeInstallation[plotID] ~= nil and aUpgradeInstallation[plotID].Given ) then
					--need to take back what was given
					DisconnectUpgradedResource( pPlot, playerID, playerTeamID )
				end

			else
				--we don't want to overwrite data if registered mine already exists on site
				if(aUpgradeInstallation[plotID] == nil) then
					--register the new mine existence
					aUpgradeInstallation[plotID] = {};
					aUpgradeInstallation[plotID].Given = false;
					aUpgradeInstallation[plotID].Owner = playerID;
					updateSaveData(plotID);

				end

			end
		end
	end
end

Events.SerialEventImprovementCreated.Add( onImprovementUpgradeCreated );

--------------------------------------------------------------

function CheckImprovementsOnTurnStart()
	
	--local playerID = Game.GetActivePlayer();
	--local playerTeamID = Game.GetActiveTeam();
	--local player = Players[playerID];

	for iPlayerLoop = 0, GameDefines.MAX_CIV_PLAYERS-1, 1 do
		
		for plotID, v in pairs(aUpgradeInstallation) do
		
			local plotY = math.floor(plotID / mapMaxX);
			local plotX = plotID - plotY * mapMaxX;
			local pPlot = Map.GetPlot( plotX, plotY );

			local playerID = aUpgradeInstallation[plotID].Owner;
			local pPlayerID = Players[playerID];
			local playerTeamID = Teams[pPlayerID:GetTeam()];
			
			local pPlayerLoop = Players[iPlayerLoop];
			--local playerLoopID = pPlayerLoop:GetID();
			local pLoopTeam = Teams[pPlayerLoop:GetTeam()];
			
			local cleanupFlag = true;

			if (pPlot ~= nil) then

				local pPlotOwner = pPlot:GetOwner();
		
				-- update status
				local improvementType = pPlot:GetImprovementType();
				local isImprovementUpgrade = improvementType == GameInfoTypes.IMPROVEMENT_ENHANCED_MINE or GameInfoTypes.IMPROVEMENT_ENHANCED_QUARRY or improvementType == GameInfoTypes.IMPROVEMENT_FAIR_TRADE_PLANTATION;
			
				if (isImprovementUpgrade) then

					if (pPlotOwner ~= -1) then
				
						if (pPlotOwner ~= playerID) then

							if (aUpgradeInstallation[plotID].Given) then
								DisconnectUpgradedResource( pPlot, playerID, playerTeamID );
							end

							cleanupFlag = true;

						else

							if (aUpgradeInstallation[plotID].Given) then
								print(aUpgradeInstallation[plotID].Given);
								ConnectUpgradedResource( pPlot, playerID, playerTeamID );
							end

						end

					else

						if (aUpgradeInstallation[plotID].Given) then
							DisconnectUpgradedResource( pPlot, playerID, playerTeamID );
						end

						cleanupFlag = true;

					end

					cleanupFlag = false;

				end
			end
		end
				
		if (cleanupFlag) then
			
			aUpgradeInstallation[plotID] = nil;
			updateSaveData(plotID);
		
		end
	end
end

Events.ActivePlayerTurnStart.Add( CheckImprovementsOnTurnStart );

--------------------------------------------------------------

function ConnectUpgradedResource( pPlot, playerID, playerTeamID )
	--checks for validity of arguments are to be performed by calling function	
	local plotID = pPlot:GetX() + pPlot:GetY() * mapMaxX;
	
	local pPlayer = Players[playerID];
	local iResourceID = pPlot:GetResourceType(playerTeamID);
	local iResourceNum = pPlot:GetNumResource();

	if ( iResourceID > -1 and iResourceNum > 0 ) then
	
		--register/update status
		aUpgradeInstallation[plotID] = {};
		aUpgradeInstallation[plotID].Resource = iResourceID;
		aUpgradeInstallation[plotID].Quantity = iResourceNum;
		aUpgradeInstallation[plotID].Given = true;
		aUpgradeInstallation[plotID].Owner = playerID;
		updateSaveData(plotID);
		
		--give resource to player
		pPlayer:ChangeNumResourceTotal(iResourceID, iResourceNum);

	end
end

--------------------------------------------------------------

function DisconnectUpgradedResource( pPlot, playerID, playerTeamID )

	local plotID = pPlot:GetX() + pPlot:GetY() * mapMaxX;
	
	local pPlayer = Players[playerID];
	local iResourceID = aUpgradeInstallation[plotID].Resource;
	local iResourceNum = aUpgradeInstallation[plotID].Quantity;

	--take resource from player
	pPlayer:ChangeNumResourceTotal(iResourceID, -iResourceNum);
	
	--update status
	aUpgradeInstallation[plotID].Given = false;
	updateSaveData(plotID);

end

--------------------------------------------------------------

function updateSaveData(plotID)
	
	local pPlayer = Players[Game.GetActivePlayer()];
	
	save( pPlayer, plotID, aUpgradeInstallation[plotID] );
	--save(plotID, aUpgradeInstallation[plotID]);

end

--------------------------------------------------------------

function loadSaveData()
	
	local pPlayer = Players[Game.GetActivePlayer()];
	local wholeTable = load( pPlayer );
	
	for plotID, v in pairs(wholeTable) do
		
		--print("plotID:",plotID);
		
		--for ii,kk in pairs(v) do
			--print(ii,kk);
		--end
		
		aUpgradeInstallation[plotID] = v;
		
	end
end

loadSaveData();	--call on startup